/*
m_ana_clap - A virtual analogue clap with many features

DESCRIPTION
This is a virtual analogue clap with control over the length of a single clap,
clap repeats, tone shaping and a built-in reverb for ringing.
NOTE: this UDO requires m_segk.udo and its accompanying ftables
log.ftable and exp.ftable . The ftables are automatically includes by this
module.

SYNTAX
aout m_ana_clap ksize, kdecay, ktone, kclaps, kdistance, kfilter, kring, \
	kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - size or pitch of the clap (0 to 1)
kdecay - Decay of a single clap (0 to 1)
ktone - brightness of the clap (0 to 1)
kclaps - Number of claps (0 to 1)
kdistance - Distance between claps (0 to 1)
kfilter - Filter on the delayed claps (0 to 1)
kring - Ringing of the clap (0 to 1)
kaccent - Accent, if >0.5 volume will be multiplied by 1.25
ktrig - Trigger input

CREDITS
Author: Jeanette C.
*/

opcode m_ana_clap, a, kkkkkkkkk
	ksize, kdec, ktone, kclaps, kgrains, kfilt, kring, kacc, ktrig xin

	; Set up the necessary ftables
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"

	; Set up real values from the 0 to 1 ranges
	kcentre = 500 + (ksize * 1500)
	ktim = .01 + (kdec * .099)
	kbright = kcentre * (1 + 5 * ktone)
	kclap_count = int(kclaps * 10)
	kdeltim = .005 + (kgrains * .045)
	kdamp = .1 + (kfilt * .9)
	krev = .3 + (.3 * kring)

	; Set up the envelopes
	kvenv, kvfin m_segk .6, ktim, 0, 0, ktrig, iflog, iflin, ifexp
	kdenv, kdfin m_segk kbright, kclap_count*kdeltim, kcentre*kdamp, 1, ktrig, iflog, iflin, ifexp ; for the LP on delays

	; Audio outputs
	afd init 0 ; Delay signnal
	anoise fractalnoise kvenv, 0 ; Basic noise

	; Delayed claps, if needed, tap delay kclap_count times
	if (kclap_count >= 1) then
		adummy delayr .5
		kndx = 0
		while (kndx <= kclap_count) do
			adly deltap (kndx + 1)*kdeltim
			afd += adly * (.1 + (.45 * kclap_count / (kndx + 1)))
			kndx += 1
		od
		delayw anoise
		afd butterlp afd*.3, kdenv
	endif

	; Reverberate for better sound
	arev nreverb anoise, krev, 1
	arev butterlp arev, kbright
	afnoise zdf_2pole ((anoise + afd)*.9 + arev*.1), kcentre, 7, 2
	afnoise zdf_2pole afnoise, kcentre*1.5, 1, 1
	ahnoise zdf_2pole anoise, kcentre*3, 2, 3
	aout = afnoise + ahnoise * (1 - (ksize * ksize))

	; If accent >0.5 amplify by 1.25
	if kacc >.5 then
		aout *= 1.25
	endif

	xout aout
endop
