/*
m_ana_kick - A virtual analogue kick with many features

DESCRIPTION
This is a virtual analogue kick with many features: basic pitch, decay, pitch
envelope, FM envelope, extra klick and filtered noise.
NOTE: this UDO requires the UDO m_segk and the accompanying ftables:
log.ftable and exp.ftable . These ftables are automatically loaded by this
module.

SYNTAX
aout m_ana_kick ksize, kdecay, kpitch_amount, kpitch_decay, kfm_amount, \
	kfm_decay, klicksize, knoise, kaccent, ktrig

INITIALIZATION

PERFORMANCE
ksize - Size of the kick drum body (0 to 1)
kdecay - Decay of the kick drum body (0 to 1)
kpitch_amount - Pitch modulation amount (0 to 1)
kpitch_decay - Pitch evenlope decay (0 to 1)
kfm_amount - FM amount (0 to 1)
kfm_decay - FM envelope decay (0 to 1)
klicksize - Size/pitch of the klick (0 to 1)
knoise - Noise amount (0 to 1)
kaccent - Accent >0.5 volume is raised by fixed amount 5/4 of normal volume
ktrig - Trigger input, if 1 kick is triggered

CREDITS
Author: Jeanette C.
*/

opcode m_ana_kick, a, kkkkkkkkkk
	ksize, klength, kpitch, katt, kmdepth, kmlength, klicksize, knoise, kacc, ktrig xin

	; Generate necessary ftables
	iflog ftgenonce 0, 0, 4096, -23, "log.ftable"
	iflin ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	ifexp ftgenonce 0, 0, 4096, -23, "exp.ftable"
	isine ftgenonce 0, 0, 32768, 10, 1

	; Set up real values from 0-1 parameter ranges
	kbasefreq = 20 + (ksize * 60) ; Basic frequency
	kdec = .01 + (klength * .39) ; Amp decay in seconds
	kpdec = .005 + (katt * .045) ; Pitch decay in seconds
	kstartpitch = kbasefreq + ((1000 - kbasefreq) * kpitch) ; maximum pitch
	kmoddepth = kmdepth * 20 ; Maximum FM mmod depth
	kmdec = kmlength * .05 ; FM decay in seconds
	klickfreq = 400 + (2000 * klicksize) ; Frequency of the klick

	; Set up envelopes for amp, pitch, FM and the klick amp
	kvenv, kvfin m_segk .6, kdec, 0, 1.5, ktrig, iflog, iflin, ifexp
	kpenv, kpfin m_segk kstartpitch, kpdec, kbasefreq, 2, ktrig, iflog, iflin, ifexp
	kmenv, kmfin m_segk kmoddepth, kmdec, 0, 1, ktrig, iflog, iflin, ifexp
	kkenv, kkfin m_segk .48, .001, 0, 2, ktrig, iflog, iflin, ifexp

	; Audio outputs: base/body of the kick, klick and noise
	abase foscil kvenv, kpenv, 1, 1, kmenv, isine
	aklick oscil kkenv, klickfreq, isine
	anoise fractalnoise kvenv*knoise, 0
	afnoise butterlp anoise, kbasefreq*4
	aout = abase + aklick + afnoise

	; Check for accent and then output
	if kacc > .5 then
		aout *= 1.25
	endif

	xout aout
endop
